
 /****************************************************************************
 * PIC32 CAN example 1
 * CAN bus formed between Explorer16 with PIC32 PIm and dsPIC Pro 4 board
 *
 ****************************************************************************/
 #include "built_in.h"
 //--------------------------------------------------------------------------
unsigned int Can_Init_Flags;
unsigned int Can_Send_Flags;
unsigned int Can_Rcv_Flags;            // can flags
unsigned int Rx_Data_Len;             // received data length
unsigned int Tx_Data_Len;             // Transmit data length
char Rx_Data[8];                      // Rx Data
char Tx_Data[8];                      // Tx Data
char Msg_Rcvd;                       // reception flag
unsigned long Rx_ID;                // Rx ID's IDs
unsigned long Tx_ID;                // Tx_ID's
//-------------------------------------------------------------------------
// reserve space for 2 buffers with 8 messages (each message is 16 bytes)
// beggining of the buffer must be 32bit aligned
char FIFObuffers[2*8*16] absolute 0xA0000000;
//-------------------------------------------------------------------------
// variables
//-------------------------------------------
unsigned char count = 0;
unsigned char AdcResult = 0;
//---------------------------------------------------------------------------
 // definitions
//--------------------------------------------


//-------------------------------------------
// Control Flags
//-------------------------------------------
struct
{
 sendMsg :1;
 Run     :1;
}Flag;
 
//----------------------------------------------------------------------
// function proto types
//----------------------------------------------------------------------
void Init(void);


//---------------------------------------
 // main code
//---------------------------------------
void main()
{
 //-----------------------------------------------------
   // Initialize mcu
 //-----------------------------------------------------
    Init();

//------------------------------------------------------
  // main code loop
//------------------------------------------------------
  while(1)
  {

   //-------------------------------------------------------------------
   // If Timer Int Set Flag send Message 0x211 STD
   //--------------------------------------------------------------------
   if(Flag.sendMsg)
   {
     count++;
     Tx_Data[0] = 0;
     Tx_Data[1] = 0;
     Tx_Data[2] = count;
     Tx_Data[3] = 0;
     Tx_Data[4] = 0;
     Tx_Data[5] = 0;
     Tx_Data[6] = 0;
     Tx_Data[7] = 0xA0;
     Tx_Data_Len = 8;
     Tx_ID = 0x211;
     CAN1Write(Tx_ID,Tx_Data,Tx_Data_Len,Can_Send_Flags);
     Flag.sendMsg = 0;
   }
   //---------------------------------------------------------------------
   // Check CAN Buffer Area for Msg Received
   //---------------------------------------------------------------------
   Msg_Rcvd = CAN1Read(&Rx_ID,Rx_Data,&Rx_Data_Len,&Can_Rcv_Flags);
   if(Msg_Rcvd)
   {
     switch (Rx_ID)
     {
       case 0x200:
       LATA = Tx_Data[2]; // display counter value on Leds
       break;
      
       case 0x201:
        asm nop;
       break;
      
       case 0x301:
        asm nop;
       break;
     } // end switch
      Msg_Rcvd = 0;
    } // end if Msg_Rcvd
  } // end while(1)
}// end main
//--------------------------------------------------------------------------
// Timer 1 ISR Handler
//--------------------------------------------------------------------------
void Timer1_interrupt() iv IVT_TIMER_1 ilevel 7 ics ICS_SRS {
  T1IF_bit = 0;           // Clear T1IF
  Flag.sendMsg = 1;
}
//--------------------------------------------------------------------------
void Init()
{
  JTAGEN_bit = 0;           // Switch JTAG port Off
  AD1PCFG = 0;              // Configure AN pins as digital I/O
  TRISA = 0;                // PORTA as outputs for Led's
  LATA = 0;                 // Switch all leds off on start up
//-------------------------------------------------------------------------
  // Configure Timer 1 Interrupt
//-------------------------------------------------------------------------
  TMR1 = 0;                 // reset timer value to zero
  PR1 = 50000;              // Load period register

  T1IP0_bit = 1;            // set interrupt
  T1IP1_bit = 1;            // priority
  T1IP2_bit = 1;            // to 7

  TCKPS0_bit = 1;           // Set Timer Input Clock
  TCKPS1_bit = 1;           // Prescale value to 1:256
  EnableInterrupts();       // Enable all interrupts

  T1IE_bit = 1;             // Enable Timer1 Interrupt
  ON__T1CON_bit = 1;        // Enable Timer1
  
//------------------------------------------------------------------------
 // Configure CAN module 1
//------------------------------------------------------------------------
  Can_Init_Flags = 0;                     //
  Can_Send_Flags = 0;                    // clear flags
  Can_Rcv_Flags  = 0;                    //

  Can_Send_Flags = _CAN_TX_STD_FRAME &   // Tx Message Format
                   _CAN_TX_NO_RTR_FRAME;
                   
  Can_Init_Flags = _CAN_CONFIG_SAMPLE_THRICE &  // form value to be used
                   _CAN_CONFIG_PHSEG2_PRG_ON &  // with CAN1Initialize
                   _CAN_CONFIG_STD_MSG &
                   _CAN_CONFIG_MATCH_MSG_TYPE &
                   _CAN_CONFIG_LINE_FILTER_OFF;
 //-------------------------------------------------------
  // set CONFIGURATION mode
 //-------------------------------------------------------
  CAN1SetOperationMode(_CAN_MODE_CONFIG,0xFF);
  
 //-------------------------------------------------------
   //initialize CAN1
 //-------------------------------------------------------
  CAN1Initialize(1,8,3,3,3,Can_Init_Flags);
  
  //------------------------------------------------------
    //assign the buffers
 //-------------------------------------------------------
  CAN1AssignBuffer(FIFObuffers);
   
 //-------------------------------------------------------
 //configure rx fifo   RX buffer 8 messages deep
 //--------------------------------------------------------
  CAN1ConfigureFIFO(0, 8,_CAN_FIFO_RX & _CAN_FULL_MESSAGE);
  
 //------------------------------------------------------------
  //configure tx fifo TX buffer 8 messages deep
 //----------------------------------------------------------------------------
  CAN1ConfigureFIFO(1,8,_CAN_FIFO_TX & _CAN_TX_PRIORITY_3 &
                    _CAN_TX_NO_RTR_FRAME);
//-----------------------------------------------------------------------------
 //set mask 0 - set all mask1 bits to ones
//-----------------------------------------------------------------------------
  CAN1SetMask(_CAN_MASK_0,-1,_CAN_CONFIG_MATCH_MSG_TYPE & 
              _CAN_CONFIG_STD_MSG);
//-----------------------------------------------------------------------------
 //set mask 1 - set all mask1 bits to ones
//-----------------------------------------------------------------------------
  CAN1SetMask(_CAN_MASK_1,-1,_CAN_CONFIG_MATCH_MSG_TYPE &
              _CAN_CONFIG_STD_MSG);
//-----------------------------------------------------------------------------
 //set mask 2 - set all mask1 bits to ones
//-----------------------------------------------------------------------------
  CAN1SetMask(_CAN_MASK_2,-1,_CAN_CONFIG_MATCH_MSG_TYPE &
              _CAN_CONFIG_STD_MSG);
//-----------------------------------------------------------------------------
 //set mask 3 - set all mask1 bits to ones
//-----------------------------------------------------------------------------
  CAN1SetMask(_CAN_MASK_3,-1,_CAN_CONFIG_MATCH_MSG_TYPE &
              _CAN_CONFIG_STD_MSG);
//-----------------------------------------------------------------------------
  //set filter 1
//-----------------------------------------------------------------------------
  CAN1SetFilter(_CAN_FILTER_31,0x200,_CAN_MASK_0,_CAN_BUFFER_0,
                 _CAN_CONFIG_STD_MSG);
//-----------------------------------------------------------------------------
 // Set CAN module in Normal Operation
//----------------------------------------------------------------------------
  CAN1SetOperationMode(_CAN_MODE_NORMAL,0xFF);
  
  Msg_Rcvd = 0; // Clear Message received Flag

}
//---------------------------------------------------------------------------